package com.fzm.admin.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.Banner;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.BannerService;
import com.fzm.common.utils.ObsUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;

/**
 * @author tangtuo
 * @date 2022/1/11 14:35
 */
@Authentication
@RestController
@RequestMapping("/banner")
@Api(tags = "banner页管理")
public class BannerController {

    @Resource
    private BannerService bannerService;

    @Resource
    private ObsUtil obsUtil;

    @PostMapping("/create")
    @ApiOperation("新建banner")
    public ResponseModel<Boolean> create(MultipartFile posterFile, String name, String jumpUrl, Integer sort) throws IOException {
        if (posterFile == null || StringUtils.isAnyBlank(name, jumpUrl) || sort == null) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }
        String url = obsUtil.putObject(posterFile);
        Banner banner = new Banner();
        banner.setName(name);
        banner.setJumpUrl(jumpUrl);
        banner.setPoster(url);
        banner.setSort(sort);
        return ResponseModel.success(bannerService.save(banner));
    }

    @GetMapping("/page")
    @ApiOperation("分页查询banner列表")
    public ResponseModel<PageInfo<Banner>> page(@RequestParam Integer pageNum, @RequestParam Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<Banner> list = bannerService.listAll();
        return ResponseModel.success(new PageInfo<>(list));
    }

    @GetMapping("/detail")
    @ApiOperation("查询banner详情")
    public ResponseModel<Banner> getDetailById(@RequestParam Integer id) {
        return ResponseModel.success(bannerService.getById(id));
    }

    @PostMapping("/update")
    @ApiOperation("修改banner")
    public ResponseModel<Boolean> update(MultipartFile posterFile, Integer id, String name, String jumpUrl, Integer sort) throws IOException {
        if (id == null) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }
        Banner banner = new Banner();
        if (posterFile != null) {
            // 删除原有海报
            String poster = bannerService.getById(id).getPoster();
            obsUtil.delete(poster);
            // 上传新海报
            String posterUrl = obsUtil.putObject(posterFile);
            banner.setPoster(posterUrl);
        }
        banner.setName(name);
        banner.setJumpUrl(jumpUrl);
        banner.setSort(sort);
        banner.setId(id);
        return ResponseModel.success(bannerService.updateById(banner));
    }

    @PostMapping("/delete")
    @ApiOperation("删除banner")
    public ResponseModel<Boolean> delete(@RequestParam Integer id) {
        return ResponseModel.success(bannerService.removeById(id));
    }
}
