package com.fzm.admin.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.vo.CopyrightApplyVo;
import com.fzm.common.entity.vo.CopyrightVo;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.CopyrightApplyService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.concurrent.ExecutionException;

/**
 * @author tangtuo
 * @date 2021/12/15 10:27
 */
@Authentication
@RestController
@RequestMapping(value = "/copyright/apply")
@Api(tags = "版权审核")
public class CopyrightApplyController {

    @Resource
    private CopyrightApplyService copyrightApplyService;


    @GetMapping(value = "/pages")
    @ApiOperation(value = "分页查询")
    public ResponseModel<PageInfo<CopyrightApplyVo>> page(@ApiParam(value = "页码", required = true) @RequestParam Integer pageNum,
                                                          @ApiParam(value = "每页记录数", required = true) @RequestParam Integer pageSize,
                                                          @ApiParam(value = "手机号") @RequestParam(required = false) String telephone,
                                                          @ApiParam(value = "流水号") @RequestParam(required = false) String serialNum,
                                                          @ApiParam(value = "审核状态") @RequestParam(required = false) Integer registerState,
                                                          @ApiParam(value = "审核开始日期 yyyy-MM-dd") @RequestParam(required = false) String from,
                                                          @ApiParam(value = "审核截止日期 yyyy-MM-dd") @RequestParam(required = false) String to) {
        PageInfo<CopyrightApplyVo> pages = copyrightApplyService.pages(pageNum, pageSize, telephone, serialNum, registerState, from, to);
        return ResponseModel.success(pages);
    }

    @PostMapping(value = "/pass")
    @ApiOperation(value = "通过")
    public ResponseModel<Boolean> pass(@RequestParam Integer id) throws ExecutionException, InterruptedException {
        boolean result = copyrightApplyService.pass(id);
        return ResponseModel.success(result);
    }

    @PostMapping(value = "/reject")
    @ApiOperation(value = "驳回")
    public ResponseModel<Boolean> reject(@RequestParam Integer id, @RequestParam String rejectReason) {
        boolean result = copyrightApplyService.reject(id, rejectReason);
        return ResponseModel.success(result);
    }

    @GetMapping(value = "/detail")
    @ApiOperation(value = "获取详情")
    public ResponseModel<CopyrightVo> details(@RequestParam Integer id) throws ExecutionException, InterruptedException {
        return ResponseModel.success(copyrightApplyService.getDetail(id));
    }
}
