package com.fzm.admin.listener;

import cn.fzm.chain.simplesdk.client.ParaChainClient;
import com.fzm.common.config.RabbitMQConfig;
import com.fzm.common.entity.CopyrightApply;
import com.fzm.common.entity.dto.EvidenceHashMessage;
import com.fzm.common.service.CopyrightApplyService;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;

/**
 * @author tangtuo
 * @date 2022/1/7 16:13
 */
@Slf4j
@Component
public class EvidenceHashLister {

    @Resource
    private CopyrightApplyService copyrightApplyService;

    @Resource
    private ParaChainClient paraChainClient;

    /**
     * 更新存证hash
     */
    @RabbitListener(queues = RabbitMQConfig.EVIDENCE_HASH_QUEUE)
    public void listenEvidenceHash(EvidenceHashMessage evidenceHashMessage, Message message, Channel channel) throws IOException {
        long deliveryTag = message.getMessageProperties().getDeliveryTag();
        try {
            String hash = evidenceHashMessage.getHash();
            String serial_code = evidenceHashMessage.getSerial_code();
            String realTxHash = paraChainClient.getRealTxHashFromGrp(hash);
            if (StringUtils.isBlank(realTxHash)) {
                log.error("存证hash查询为空, 交易hash为: {}, 流水号为: {}", hash, serial_code);
                // 查询失败,拒绝签收
                channel.basicReject(deliveryTag, true);
                return;
            }
            CopyrightApply apply = copyrightApplyService.getBySerialNum(serial_code);
            apply.setEvidenceHash(realTxHash);
            copyrightApplyService.updateById(apply);
            // 手动签收
            channel.basicAck(deliveryTag, false);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            // 发生异常, 拒收消息
            channel.basicReject(deliveryTag, true);
        }


    }
}
