package com.fzm.common.entity;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * @author tangtuo
 * @date 2021/12/13 11:22
 */
@Data
@AllArgsConstructor
@TableName(value = "tb_copyright_apply_owner_relation")
public class CopyrightApplyOwnerRelation {

    @TableField(value = "copyright_id")
    @ApiModelProperty(value = "版权id")
    private Integer copyrightId;

    @TableField(value = "owner_id")
    @ApiModelProperty(value = "著作权人id")
    private Integer ownerId;

    @NotNull(message = "著作权人类型不能为空")
    @ApiModelProperty(value = "类别 0-自然人  1-法人")
    private Integer type;

    @NotBlank(message = "著作权人不能为空")
    @ApiModelProperty(value = "著作权人")
    private String owner;

    @NotBlank(message = "证件号码不能为空")
    @ApiModelProperty(value = "证件号码")
    private String idNumber;

    @NotNull(message = "证件是否长期有效不能为空")
    @ApiModelProperty(value = "是否长期有效 0-否 1-是")
    private Integer isEffective;

    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @NotNull(message = "证件有效期起始时间不能为空")
    @ApiModelProperty(value = "证件有效起止时间")
    private Date effectiveStartDate;

    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @ApiModelProperty(value = "证件有效终止时间")
    private Date effectiveEndDate;

    @NotBlank(message = "用户所在省份不能为空")
    @ApiModelProperty(value = "用户所在省份")
    private String province;

    @ApiModelProperty(value = "用户所在省份code")
    private String provinceCode;

    @NotBlank(message = "用户所在城市不能为空")
    @ApiModelProperty(value = "用户所在城市")
    private String city;

    @ApiModelProperty(value = "用户所在城市code")
    private String cityCode;

    @ApiModelProperty(value = "证件类型  0-身份证 1-营业执照  2-企业法人营业执照  3-组织机构代码证书  4-事业单位法人证书  5-社团法人证书  6-统一社会信用代码  7其他有效证件")
    private Integer idType;

    @ApiModelProperty(value = "正面照片 (身份证人像面照片)")
    private String positivePhoto;

    @ApiModelProperty(value = "反面照片(身份证国徽面照片)")
    private String backPhoto;

    @ApiModelProperty(value = "手持身份证照片")
    private String personalPhoto;

    @ApiModelProperty(value = "证件照片(营业执照等)")
    private String certificatesPhoto;

    @NotBlank(message = "委托证明不能为空")
    @ApiModelProperty(value = "委托证明")
    private String entrustProve;

    private Date createDate;

    private Date updateDate;

    public CopyrightApplyOwnerRelation(Integer copyrightId,CopyrightOwner copyrightOwner) {
        this.copyrightId = copyrightId;
        this.ownerId = copyrightOwner.getId();
        BeanUtil.copyProperties(copyrightOwner, this, true);
    }
}
