package com.fzm.common.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fzm.common.constant.TokenConstant;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;

import java.util.Date;

/**
 * @author tangtuo
 * @date 2021/6/23 11:54
 */
@Data
@Accessors(chain = true)
@ApiModel("用户信息")
@TableName("tb_user")
public class User extends AbstractUser {

    @TableId(type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty("手机号码(用户名)")
    private String telephone;

    @ApiModelProperty("邮箱")
    private String email;

    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    @ApiModelProperty("密码")
    private String password;

    @ApiModelProperty("钱包地址")
    private String wallet;

    @Length(max = 20, message = "昵称长度不能超过20")
    @ApiModelProperty("昵称")
    private String nickname;

    @ApiModelProperty("个性签名")
    private String signature;

    @ApiModelProperty("头像")
    private String avatar;

    @ApiModelProperty("认证类型  0-个人认证  1-企业认证")
    private Integer authType;

    @ApiModelProperty("实名认证状态  0-未认证  1-认证成功  2-认证失败")
    private Integer authStatus;

    @ApiModelProperty("是否发行过nft作品  0-否  1-是")
    private Integer isPublish;

    @ApiModelProperty("创建时间")
    private Date createDate;

    @ApiModelProperty("修改时间")
    private Date updateDate;

    @ApiModelProperty("记录创建的时间戳")
    @TableField(fill = FieldFill.INSERT)
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private Long createTime;

    @ApiModelProperty("记录更新的时间戳")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private Long updateTime;

    @ApiModelProperty("个人实名认证信息")
    @TableField(exist = false)
    private AuthPerson authPerson;

    @Override
    public String appId() {
        return TokenConstant.TOKEN_APP_ID_PORTAL;
    }

    @Override
    public Integer userId() {
        return id;
    }
}
