package com.fzm.common.entity.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/12/9 14:35
 */
@Data
public class CopyrightDTO {

    @ApiModelProperty("主键")
    private Integer id;

    @ApiModelProperty("草稿id")
    private Integer draftId;

    @NotBlank(message = "nft哈希不能为空")
    @ApiModelProperty(value = "nft哈希")
    private String nftHash;

    @NotBlank(message = "nft名称不能为空")
    @ApiModelProperty(value = "nft名称")
    private String nftName;

    @NotBlank(message = "登记委托书不能为空")
    @ApiModelProperty(value = "登记委托书")
    private String registerEntrust;

    @NotBlank(message = "作品名称不能为空")
    @ApiModelProperty(value = "作品名称")
    private String opusName;

    @NotNull(message = "作品类别id不能为空")
    @ApiModelProperty(value = "作品类别id")
    private Integer opusCategoryId;

    @NotBlank(message = "作品性质不能为空")
    @ApiModelProperty(value = "作品性质")
    private String opusProperty;

    @NotBlank(message = "内容简介不能为空")
    @ApiModelProperty(value = "内容简介")
    private String contentSynopsis;

    @NotBlank(message = "创作过程不能为空")
    @ApiModelProperty(value = "创作过程")
    private String createProcess;

    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @NotNull(message = "作品完成日期不能为空")
    @ApiModelProperty(value = "作品完成日期")
    private Date opusCompleteDate;

    @NotBlank(message = "作品完成省份不能为空")
    @ApiModelProperty(value = "作品完成省份")
    private String opusCompleteProvince;

    @NotBlank(message = "作品完成城市不能为空")
    @ApiModelProperty(value = "作品完成城市")
    private String opusCompleteCity;

    @NotNull(message = "发表状态不能为空")
    @ApiModelProperty(value = "'发表状态  0-未发表  1-已发表")
    private Integer publishState;

    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @ApiModelProperty(value = "首次发表日期 格式:yyyy-MM-dd")
    private Date firstPublishDate;

    @ApiModelProperty(value = "首次发表省份")
    private String firstPublishProvince;

    @ApiModelProperty(value = "首次发表城市")
    private String firstPublishCity;

    @NotBlank(message = "权力取得方式不能为空")
    @ApiModelProperty(value = "权力取得方式")
    private String authorityAcquireMode;

    @NotBlank(message = "权力取得证明材料不能为空")
    @ApiModelProperty(value = "权力取得证明材料")
    private String authorityAcquireProve;

    @NotBlank(message = "权力归属方式不能为空")
    @ApiModelProperty(value = "权力归属方式")
    private String authorityAscriptionMode;

    @ApiModelProperty(value = "权力归属证明材料的文件地址")
    private String authorityAscriptionProve;

    @NotBlank(message = "作品权力保证书不能为空")
    @ApiModelProperty(value = "作品权力保证书")
    private String opusPowerGuarantee;

    @NotEmpty(message = "作者列表不能为空")
    @ApiModelProperty(value = "作者列表")
    private List<Author> authors;

    @NotEmpty(message = "您还未选择任何著作权人")
    @ApiModelProperty(value = "著作权人id列表")
    private List<Integer> ownerIds;

    @NotEmpty(message = "您还未勾选任何权力拥有情况")
    @ApiModelProperty("权力id集合")
    private List<Integer> authorityIds;

    @ApiModelProperty("json串")
    private String content;

    @NotEmpty(message = "您还没有上传任何文件作品")
    @ApiModelProperty("文件列表")
    private List<File> files;

    @Data
    public static class File {

        @NotBlank(message = "附件名不能为空")
        @ApiModelProperty("附件名")
        private String fileName;

        @NotBlank(message = "附件路径不能为空")
        @ApiModelProperty("附件路径")
        private String fileUrl;

        @NotBlank(message = "文件hash不能为空")
        @ApiModelProperty("文件hash")
        private String fileHash;

        @NotBlank(message = "文件扩展名不能为空")
        @ApiModelProperty("文件扩展名")
        private String fileSuffix;

        @NotBlank(message = "文件大小不能为空")
        @ApiModelProperty("文件大小")
        private long fileSize;
    }

    @Data
    public static class Author {

        @NotBlank(message = "作者姓名不能为空")
        @ApiModelProperty(value = "作者姓名或名称")
        private String name;

        @NotBlank(message = "署名类型不能为空")
        @ApiModelProperty(value = "署名类型  (本名 | 别名 | 匿名)")
        private String signType;

        @NotBlank(message = "签名不能为空")
        @ApiModelProperty(value = "签名")
        private String sign;

    }


}
