/**
 * Copyright 2021 bejson.com
 */
package com.fzm.common.entity.dto;

import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONUtil;
import com.fzm.common.entity.*;
import com.fzm.common.entity.vo.CopyrightVo;
import com.fzm.common.enums.*;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Auto-generated: 2021-12-15 16:10:11
 *
 * @author bejson.com (i@bejson.com)
 * @website http://www.bejson.com/java2pojo/
 */
@Data
public class CopyrightRequest {

    private String app_id;
    private String charset = "utf-8";
    private String sign;
    private String sign_type = "md5";
    private String timestamp = DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss");
    private String version = "1.0";
    private String biz_content;

    @Data
    public static class CopyrightDetail {

        @ApiModelProperty(value = "流水号")
        private String serial_code;

        @ApiModelProperty(value = "权利归属方式名称图片")
        private String affilia_photo;

        @ApiModelProperty(value = "代理证明图片")//无
        private String agency_photo;

        @ApiModelProperty(value = "作者身份证明图片")
        private String author_photo;

        @ApiModelProperty(value = "合作作者")
        private List<Cooperator> cooperator = new ArrayList<>();

        @ApiModelProperty(value = "作品完成地点")
        private String end_address;

        @ApiModelProperty(value = "作品完成时间,格式：2021-09-10")
        private String end_time;

        @ApiModelProperty(value = "委托证明图片")
        private String entrust_photo;

        @ApiModelProperty(value = "权利获得方式编码")
        private Integer gain_code;

        @ApiModelProperty(value = "权利取得方式说明")
        private String gain_desc;

        @ApiModelProperty(value = "权利获得方式名称")
        private String gain_name;

        @ApiModelProperty(value = "证明材料图片")
        private String gain_photo;

        @ApiModelProperty(value = "作者名")
        private String geren_uname;

        @ApiModelProperty(value = "作者署名")
        private String geren_uxname;

        @ApiModelProperty(value = "作者署名类别")
        private String geren_uxname_type;

        @ApiModelProperty(value = "作者署名类别名称")
        private String geren_uxname_type_name;

        @ApiModelProperty(value = "是否评级（1.愿意,0不愿意）")
        private Integer is_grade;

        @ApiModelProperty(value = "作品创作性质编码")
        private Integer nature_code;

        @ApiModelProperty(value = "作品性质说明")
        private String nature_desc;

        @ApiModelProperty(value = "创作性质名称")
        private String nature_name;

        @ApiModelProperty(value = "其他证明材料图片")
        private String other_photo;

        @ApiModelProperty(value = "著作权人列表")
        private List<Owner> owner_list = new ArrayList<>();
        ;

        @ApiModelProperty(value = "发表地点")
        private String publish_address;

        @ApiModelProperty(value = "作品是否发表 发表状态（1：未发表，2：已发表）")
        private int publish_status;

        @ApiModelProperty(value = "发表时间,格式：2021-09-10")
        private String publish_time;

        @ApiModelProperty(value = "权利归属方式编码")
        private int rights_affilia_code;

        @ApiModelProperty(value = "权利归属方式名称")
        private String rights_affilia_name;

        @ApiModelProperty(value = "权利拥有说明")
        private String rights_desc;

        @ApiModelProperty(value = "权利保证书(代理人)图片")//无
        private String rights_guarantee_photo;

        @ApiModelProperty(value = "创作过程")
        private String works_create_desc;

        @ApiModelProperty(value = "作品简介")
        private String works_desc;

        @ApiModelProperty(value = "作品上传的文件")
        private List<WorksFile> works_files = new ArrayList<>();

        @ApiModelProperty(value = "作品名称")
        private String works_name;

        @ApiModelProperty(value = "作品类型编码")
        private String works_type_code;

        @ApiModelProperty(value = "作品类别说明")
        private String works_type_desc;

        @ApiModelProperty(value = "作品类别名称")
        private String works_type_name;

        public CopyrightDetail(CopyrightVo copyrightVo, OpusCategory opusCategory) {
            this.serial_code = copyrightVo.getSerialNum();
            this.affilia_photo = copyrightVo.getAuthorityAscriptionProve();
            this.agency_photo = copyrightVo.getRegisterEntrust();
            this.author_photo = "";
            this.end_address = copyrightVo.getOpusCompleteProvince() + copyrightVo.getOpusCompleteCity();
            this.end_time = DateUtil.format(copyrightVo.getOpusCompleteDate(), "yyyy-MM-dd");
            this.entrust_photo = "";
            this.gain_code = AuthorityAcquire.getByType(copyrightVo.getAuthorityAcquireMode());
            this.gain_name = copyrightVo.getAuthorityAcquireMode();
            this.gain_photo = "";
            if (copyrightVo.getAuthorityAscriptionMode().equals(AuthorityType.PERSONAL.getVal())) {
                CopyrightAuthor author = copyrightVo.getAuthors().get(0);
                this.geren_uname = author.getName();
                this.geren_uxname = author.getSign();
                this.geren_uxname_type = String.valueOf(SignatureType.getCode(author.getSignType()));
                this.geren_uxname_type_name = author.getSignType();
            } else {
                for (CopyrightAuthor author : copyrightVo.getAuthors()) {
                    this.cooperator.add(new Cooperator(author));
                }
            }
            this.is_grade = 0;
            this.nature_code = OpusProperty.getCode(copyrightVo.getOpusProperty());
            this.nature_desc = "";
            this.nature_name = copyrightVo.getOpusProperty();
            this.other_photo = "";
            for (CopyrightApplyOwnerRelation copyrightOwner : copyrightVo.getOwners()) {
                this.owner_list.add(new Owner(copyrightOwner));
            }
            this.publish_status = copyrightVo.getPublishState() + 1;
            if (this.publish_status == 2) {
                this.publish_address = copyrightVo.getFirstPublishProvince() + copyrightVo.getFirstPublishCity();
                this.publish_time = DateUtil.format(copyrightVo.getFirstPublishDate(), "yyyy-MM-dd");
            }
            this.rights_affilia_code = AuthorityType.getByType(copyrightVo.getAuthorityAscriptionMode());
            this.rights_affilia_name = copyrightVo.getAuthorityAscriptionMode();
            this.rights_desc = StringUtils.join(
                    copyrightVo.getAuthorities().stream().map(
                            Authority::getId).collect(
                            Collectors.toList()), "、");
            this.rights_guarantee_photo = copyrightVo.getOpusPowerGuarantee();
            this.works_create_desc = copyrightVo.getCreateProcess();
            this.works_desc = copyrightVo.getContentSynopsis();
            for (CopyrightFile file : copyrightVo.getFiles()) {
                this.works_files.add(new WorksFile(file));
            }
            this.works_name = copyrightVo.getOpusName();
            this.works_type_code = String.valueOf(opusCategory.getId());
            this.works_type_desc = "";
            this.works_type_name = opusCategory.getValue();
        }
    }


    @Data
    public static class Cooperator {

        @ApiModelProperty(value = "匿名类型")
        private String signature_type;

        @ApiModelProperty(value = "署名情况")
        private String signature_type_name;

        @ApiModelProperty(value = "作者姓名或名称")
        private String uname;

        @ApiModelProperty(value = "作品署名")
        private String uxname;

        public Cooperator(CopyrightAuthor author) {
            this.signature_type = String.valueOf(SignatureType.getCode(author.getSignType()));
            this.uname = author.getName();
            this.uxname = author.getSign();
            this.signature_type_name = author.getSignType();
        }
    }

    @Data
    public static class Owner {

        @ApiModelProperty(value = "地址")
        private String address;

        @ApiModelProperty(value = "身份证反面")
        private String back_photo;

        @ApiModelProperty(value = "所在市")
        private String city;

        @ApiModelProperty(value = "市code")
        private String city_id;

        @ApiModelProperty(value = "企业证件图片")
        private String company_photo;

        @ApiModelProperty(value = "证件编码")
        private String document_code;

        @ApiModelProperty(value = "其他证件照片(自然人选其他证件时上传)")
        private String document_image;

        @ApiModelProperty(value = "证件类别code(1：省内身份证)")
        private int document_type_code;

        @ApiModelProperty(value = "证件类型名称")
        private String document_type_name;

        @ApiModelProperty(value = "身份证正面")
        private String front_photo;

        @ApiModelProperty(value = "手持身份证照片")
        private String handheld_photo;

        @ApiModelProperty(value = "证件是否永久有效（1：是， 2：否）")
        private int is_forever;

        @ApiModelProperty(value = "身份证号码（选其他证件时填写）")
        private String other_document_code;

        @ApiModelProperty(value = "身份证号码（选其他证件时填写）")
        private String other_end_date;

        @ApiModelProperty(value = "著作权人姓名")
        private String owner_name;

        @ApiModelProperty(value = "署名")
        private String owner_sign_name;

        @ApiModelProperty(value = "著作权人类别(1：自然人)")
        private String owner_type_code;

        @ApiModelProperty(value = "著作权人类型名称")
        private String owner_type_name;

        @ApiModelProperty(value = "证明图片")
        private String prove_photo;

        @ApiModelProperty(value = "所在省")
        private String province;

        @ApiModelProperty(value = "省code")
        private String province_id;

        @ApiModelProperty(value = "证件有效期结束日期，格式：yyyy-MM-dd")
        private String valid_end_date;

        @ApiModelProperty(value = "证件有效期开始日期，格式：yyyy-MM-dd")
        private String valid_start_date;

        public Owner(CopyrightApplyOwnerRelation copyrightOwner) {
            this.address = copyrightOwner.getProvince() + copyrightOwner.getCity();
            this.back_photo = copyrightOwner.getBackPhoto();
            this.city = copyrightOwner.getCity();
            this.city_id = copyrightOwner.getCityCode();
            this.company_photo = copyrightOwner.getCertificatesPhoto();
            this.document_code = copyrightOwner.getIdNumber();
            this.document_image = "";
            CertificatesType certificatesType = CertificatesType.getByType(copyrightOwner.getIdType());
            this.document_type_code = certificatesType.getCode();
            this.document_type_name = certificatesType.getValue();
            this.front_photo = copyrightOwner.getPositivePhoto();
            this.handheld_photo = copyrightOwner.getPersonalPhoto();
            this.is_forever = copyrightOwner.getIsEffective() == 1 ? 1 : 2;
            this.owner_name = copyrightOwner.getOwner();
            this.owner_sign_name = "";
            this.owner_type_code = String.valueOf(copyrightOwner.getType() + 1);
            this.owner_type_name = OwnerType.getByType(copyrightOwner.getCopyrightId());
            this.prove_photo = copyrightOwner.getEntrustProve();
            this.province = copyrightOwner.getProvince();
            this.province_id = copyrightOwner.getProvinceCode();
            this.valid_start_date = DateUtil.format(copyrightOwner.getEffectiveStartDate(), "yyyy-MM-dd");
            if (this.is_forever == 2) {
                this.valid_end_date = DateUtil.format(copyrightOwner.getEffectiveEndDate(), "yyyy-MM-dd");
            }
        }
    }


    @Data
    public static class WorksFile {

        @ApiModelProperty(value = "文件")
        private String file;

        @ApiModelProperty(value = "文件格式")
        private String file_format;

        @ApiModelProperty(value = "文件名称")
        private String file_name;

        @ApiModelProperty(value = "文件大小")
        private String file_size;

        @ApiModelProperty(value = "文件token")
        private String file_token;

        @ApiModelProperty(value = "关键词")
        private String keywords;

        public WorksFile(CopyrightFile file) {
            this.file = file.getFileUrl();
            this.file_format = file.getFileSuffix();
            this.file_name = file.getFileName();
            this.file_size = String.valueOf(file.getFileSize());
            this.file_token = file.getFileHash();
            this.keywords = "";
        }
    }


    public CopyrightRequest(CopyrightVo copyrightVo, OpusCategory category, String app_id, String sign) {
        this.biz_content = JSONUtil.toJsonStr(new CopyrightDetail(copyrightVo, category));
        this.app_id = app_id;
        this.sign = sign;
    }
}