package com.fzm.common.entity.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fzm.common.entity.CopyrightApplyOwnerRelation;
import com.fzm.common.entity.CopyrightAuthor;
import com.fzm.common.entity.Nft;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.stream.Collectors;

/**
 * @author tangtuo
 * @date 2021/7/1 10:35
 */
@Data
public class NftVo {

    @ApiModelProperty("主键")
    private Integer id;

    @ApiModelProperty("存证名称")
    private String name;

    @ApiModelProperty("nft编号")
    private String nftId;

    @ApiModelProperty("存证哈希")
    private String nftHash;

    @ApiModelProperty("发行个数")
    private Integer count = 1;

    @ApiModelProperty("发行状态")
    private String publishStatus = "已发行";

    @ApiModelProperty("存证人")
    private String evidencer;

    @ApiModelProperty("作者")
    private String author;

    @ApiModelProperty("简介")
    private String synopsis;

    @ApiModelProperty("文件哈希")
    private String fileHash;

    @ApiModelProperty(value = "存证时间")
    private Date evidenceTime;

    @ApiModelProperty("版权详情")
    private Copyright copyright;

    @ApiModelProperty("持有人用户id")
    private Integer userId;


    @Data
    @NoArgsConstructor
    public static class Copyright {
        @ApiModelProperty("存证哈希")
        private String evidenceHash;

        @ApiModelProperty("存证人")
        private String evidencer = "湖北版权局";

        @ApiModelProperty("登记号")
        private String registerCode;

        @ApiModelProperty("作品名称")
        private String opusName;

        @ApiModelProperty("作品类型")
        private String opusType;

        @ApiModelProperty("作者")
        private String author;

        @ApiModelProperty("著作权人")
        private String copyrightOwner;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty("作品完成时间")
        private Date opusCompleteDate;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty("作品首次发表时间")
        private Date firstPublishDate;

        @ApiModelProperty("登记时间")
        private Date registerDate;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty("存证时间")
        private Date evidenceTime;

        private Integer registerState;


        public Copyright(CopyrightVo copyrightVo) {
            this.registerCode = copyrightVo.getRegisterCode();
            this.opusName = copyrightVo.getOpusName();
            this.opusType = copyrightVo.getOpusCategory().getValue();
            this.author = copyrightVo.getAuthors().stream().map(CopyrightAuthor::getName).collect(Collectors.joining());
            this.copyrightOwner = copyrightVo.getOwners().stream().map(CopyrightApplyOwnerRelation::getOwner).collect(Collectors.joining());
            this.opusCompleteDate = copyrightVo.getOpusCompleteDate();
            this.firstPublishDate = copyrightVo.getFirstPublishDate();
            this.evidenceTime = copyrightVo.getEvidenceDate();
            this.evidenceHash = copyrightVo.getEvidenceHash();
            this.registerState = copyrightVo.getRegisterState();
        }

    }

    public NftVo(Nft nft, CopyrightVo copyrightVo) {
        this.id = nft.getId();
        this.evidencer = nft.getEvidencer();
        this.author = nft.getAuthor();
        this.userId = nft.getUserId();
        this.name = nft.getName();
        this.nftId = nft.getNftId();
        this.nftHash = nft.getNftHash();
        this.fileHash = nft.getFileHash();
        this.evidenceTime = nft.getPublishTime();
        this.synopsis = nft.getSynopsis();
        this.copyright = (copyrightVo == null) ? null : new Copyright(copyrightVo);
    }

}
