package com.fzm.common.exception.handler;

import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.validation.BindException;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ：tangtuo
 * @date ：2021/04/30 16:35
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {


    @ExceptionHandler(value = GlobalException.class)
    public ResponseModel<String> handlerGlobalException(GlobalException globalException) {
        log.error(globalException.getMessage(), globalException);
        return ResponseModel.fail(globalException.getErrorCode(), globalException.getMessage());
    }

    @ExceptionHandler(value = Exception.class)
    public ResponseModel<String> handlerException(Exception exception) {
        log.error(exception.getMessage(), exception);
        return ResponseModel.fail(exception);
    }


    @ExceptionHandler(value = {MethodArgumentNotValidException.class})
    public ResponseModel<String> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
        log.error(e.getMessage(), e);
        List<ObjectError> allErrors = e.getBindingResult().getAllErrors();
        String errorMsg = allErrors.
                stream().
                map(DefaultMessageSourceResolvable::getDefaultMessage).
                collect(Collectors.joining("、"));
        return ResponseModel.fail(ResultCode.VALIDATE_FAILED, errorMsg);
    }

    @ExceptionHandler(value = {BindException.class})
    public ResponseModel<String> handleBindException(BindException e) {
        log.error(e.getMessage(), e);
        List<ObjectError> allErrors = e.getBindingResult().getAllErrors();
        String errorMsg = allErrors.
                stream().
                map(DefaultMessageSourceResolvable::getDefaultMessage).
                collect(Collectors.joining("、"));
        return ResponseModel.fail(ResultCode.VALIDATE_FAILED, errorMsg);
    }
}
