package com.fzm.common.model;


import com.fzm.common.enums.IErrorCode;
import com.fzm.common.enums.ResultCode;
import lombok.Data;

/**
 * @author ：tangtuo
 * @date ：2021/04/30 16:03
 */
@Data
public class ResponseModel<T> {

    private int code;
    private String message;
    private T data;

    public ResponseModel() {
    }

    private ResponseModel(IErrorCode errorCode, T data) {
        this.code = errorCode.getCode();
        this.message = errorCode.getMessage();
        this.data = data;
    }

    public static <T> ResponseModel<T> success(T data) {
        return new ResponseModel<>(ResultCode.SUCCESS, data);
    }

    public static <T>  ResponseModel<T> success() {
        return new ResponseModel<>(ResultCode.SUCCESS, null);
    }

    public static <T> ResponseModel<T> fail() {
        return new ResponseModel<>(ResultCode.FAILED, null);
    }

    public static <T> ResponseModel<T> fail(T data) {
        return new ResponseModel<>(ResultCode.FAILED, data);
    }

    public static <T> ResponseModel<T> fail(IErrorCode errorCode) {
        return new ResponseModel<>(errorCode, null);
    }

    public static ResponseModel<String> fail(IErrorCode errorCode, String errorMsg) {
        return new ResponseModel<>(errorCode, errorMsg);
    }

    public static ResponseModel<String> fail(Throwable throwable) {
        return fail(throwable.getMessage());
    }

}
