package com.fzm.common.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fzm.common.entity.CopyrightApply;
import com.fzm.common.entity.dto.CopyrightDTO;
import com.fzm.common.entity.vo.CopyrightApplyVo;
import com.fzm.common.entity.vo.CopyrightCertificateVo;
import com.fzm.common.entity.vo.CopyrightVo;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.concurrent.ExecutionException;

/**
 * @author tangtuo
 * @date 2021/12/9 11:40
 */
public interface CopyrightApplyService extends IService<CopyrightApply> {

    /**
     * 提交版权申请
     *
     * @param copyrightDTO
     * @return
     */
    Integer submit(CopyrightDTO copyrightDTO);

    /**
     * 查询用户的版权登记列表
     *
     * @param userId
     * @param state
     * @return
     */
    List<CopyrightApply> list(Integer userId, Integer state);

    /**
     * 撤回
     *
     * @param id
     * @return
     */
    Boolean withdraw(Integer id);


    /**
     * 删除记录
     *
     * @param id
     * @return
     */
    Boolean delete(Integer id);

    /**
     * 获取详情
     *
     * @param id
     * @return
     */
    CopyrightVo getDetail(Integer id) throws ExecutionException, InterruptedException;

    /**
     * 修改
     *
     * @param copyrightDTO
     */
    Boolean update(CopyrightDTO copyrightDTO);

    /**
     * 分页查询
     *
     * @param pageNum
     * @param pageSize
     * @param telephone
     * @param serialNum
     * @param registerState
     * @param from
     * @param to
     * @return
     */
    PageInfo<CopyrightApplyVo> pages(Integer pageNum, Integer pageSize, String telephone, String serialNum, Integer registerState, String from, String to);

    /**
     * 驳回
     *
     * @param id
     * @param rejectReason
     * @return
     */
    boolean reject(Integer id, String rejectReason);

    /**
     * 审核通过
     *
     * @param id
     * @return
     */
    boolean pass(Integer id) throws ExecutionException, InterruptedException;

    /**
     * 获取存证成功的总数量
     *
     * @return
     */
    int getSuccessCount();

    /**
     * 根据流水号查询
     *
     * @param serial_code
     * @return
     */
    CopyrightApply getBySerialNum(String serial_code);

    /**
     * 根据nft查询详情
     *
     * @param nftHash
     * @return
     */
    CopyrightVo getByNftHash(String nftHash) throws ExecutionException, InterruptedException;

    /**
     * 查看证书
     *
     * @param id
     * @return
     */
    CopyrightCertificateVo getCertificate(Integer id);

    /**
     * 判断当前nft是否申请过版权
     *
     * @param nftHash
     * @return
     */
    Boolean existsByNftHash(String nftHash);

    /**
     * 查询所有待更新状态的版权申请的流水号
     *
     * @return
     */
    List<String> getSerialCodes();

}
