package com.fzm.common.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fzm.common.entity.Nft;
import com.fzm.common.entity.NftDto;
import com.fzm.common.entity.vo.CollectionNftVo;
import com.fzm.common.entity.vo.NftCertificateVo;
import com.fzm.common.entity.vo.NftListVo;
import com.fzm.common.entity.vo.NftVo;
import com.fzm.common.params.NftTransferParam;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.concurrent.ExecutionException;

/**
 * @author tangtuo
 * @date 2021/6/30 15:55
 */
public interface NftService extends IService<Nft> {
    /**
     * 保存nft基本信息
     *
     * @param nft
     * @return
     */
    NftDto saveNft(Nft nft);

    /**
     * 查看nft列表
     *
     * @param pageNum
     * @param pageSize
     * @param categoryId
     * @return
     */
    List<Nft> list(Integer pageNum, Integer pageSize, Integer categoryId);

    /**
     * 收藏nft
     *
     * @param id nft主键
     * @return
     */
    Boolean collection(Integer id);

    /**
     * 查询用户的收藏列表
     *
     * @return
     */
    List<CollectionNftVo> getCollectionList();

    /**
     * 查询我的nft列表
     *
     * @param categoryId
     * @param userId
     * @return
     */
    List<CollectionNftVo> listCurrent(Integer categoryId, Integer userId);

    /**
     * 生成nft编号
     *
     * @param categoryId
     * @return
     */
    String generateNftId(Integer categoryId);

    /**
     * 后台管理系统根据条件检索nft列表
     *
     * @param pageNum
     * @param pageSize
     * @param categoryId
     * @param name
     * @param telephone
     * @param status
     * @param start
     * @param end
     * @return
     */
    PageInfo<NftListVo> page(Integer pageNum, Integer pageSize, Integer categoryId, String name, String telephone, Integer status, String start, String end);

    /**
     * 置顶
     *
     * @param id
     * @return
     */
    Boolean top(Integer id);

    /**
     * 取消置顶
     *
     * @param id
     * @return
     */
    Boolean untop(Integer id);

    /**
     * 上架
     *
     * @param id
     * @return
     */
    Boolean onShelf(Integer id);

    /**
     * 下架
     *
     * @param id
     * @return
     */
    Boolean offShelf(Integer id);

    /**
     * 下载附件
     *
     * @param id
     */
    void download(Integer id);

    /**
     * 发行nft
     *
     * @param nftDto
     * @return
     */
    Integer publish(NftDto nftDto);

    /**
     * nft转让
     *
     * @param param
     * @return
     */
    Boolean transfer(NftTransferParam param);

    /**
     * 根据nft哈希查询
     *
     * @param nftHash
     * @return
     */
    Nft getByNftHash(String nftHash);

    /**
     * 查询用户还未申请版权的nft列表
     *
     * @param userId
     * @return
     */
    List<CollectionNftVo> listCopyright(Integer userId);

    /**
     * 根据tokenId查询nft
     *
     * @param tokenId
     * @return
     */
    Nft getByTokenId(Long tokenId);

    /**
     * 获取nft详情
     *
     * @param nftHash
     * @return
     */
    NftVo getDetail(String nftHash) throws ExecutionException, InterruptedException;

    /**
     * 修改当前nfthash的是否已委托的状态
     *
     * @param nftHash
     * @param isEntrust
     */
    void updateEntrust(String nftHash, Integer isEntrust);
}
