package com.fzm.common.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fzm.common.entity.AuthPerson;
import com.fzm.common.entity.User;
import com.fzm.common.entity.vo.UserListVo;
import com.fzm.common.entity.vo.UserStatisticVo;
import com.fzm.common.params.LoginParam;
import com.tencentcloudapi.faceid.v20180301.models.IdCardOCRVerificationResponse;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/6/23 15:14
 */
public interface UserService extends IService<User> {

    /**
     * 根据手机号查询用户信息
     *
     * @param username
     * @return
     */
    User loadUserByUsername(String username);

    /**
     * 登录
     *
     * @param param
     * @return
     */
    User login(LoginParam param);

    /**
     * 根据token获取用户信息
     *
     * @return
     */
    User getUserByToken();

    /**
     * 修改头像
     *
     * @param avatar
     * @return
     */
    User uploadAvatar(String avatar);

    /**
     * 查询用户统计概览信息
     *
     * @return
     */
    UserStatisticVo getStatistic();

    /**
     * @param telephone
     * @param nickname
     * @param authType
     * @param start
     * @param end
     * @return
     */
    List<UserListVo> list(String telephone, String nickname, Integer authType, String start, String end);

    /**
     * 根据钱包地址查询用户信息
     *
     * @param publishAddress
     * @return
     */
    User getUserByWallet(String publishAddress);

    IdCardOCRVerificationResponse personAuth(MultipartFile cardPictureFront, MultipartFile cardPictureBack) throws IOException;

    AuthPerson getAuthDetail(Integer userId);
}
