package com.fzm.common.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.entity.CopyrightApply;
import com.fzm.common.entity.Draft;
import com.fzm.common.entity.dto.CopyrightDTO;
import com.fzm.common.entity.dto.DraftDTO;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.DraftMapper;
import com.fzm.common.service.*;
import com.fzm.common.utils.ObsUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/12/10 10:18
 */
@Service
public class DraftServiceImpl extends ServiceImpl<DraftMapper, Draft> implements DraftService {

    @Resource
    private DraftMapper draftMapper;

    @Resource
    private ObsUtil obsUtil;

    @Resource
    private CopyrightApplyService copyrightApplyService;

    @Resource
    private CopyrightFileService copyrightFileService;

    @Resource
    private CopyrightAuthorityRelationService copyrightAuthorityRelationService;

    @Resource
    private CopyrightAuthorService copyrightAuthorService;

    @Override
    public List<Draft> getByUserId(Integer userId) {
        QueryWrapper<Draft> wrapper = new QueryWrapper<>();
        wrapper.eq("user_id", userId);
        return draftMapper.selectList(wrapper);
    }

    @Override
    public Boolean delete(Integer id) {
        Draft draft = getById(id);
        if (draft == null) {
            throw GlobalException.newException(ResultCode.FAILED, "删除草稿失败,此记录不存在");
        }
        CopyrightDTO copyrightDTO = JSONUtil.toBean(draft.getJson(), CopyrightDTO.class);
        if (copyrightDTO != null) {
            // 删除权力取得方式的附件权力
            if (StringUtils.isNotBlank(copyrightDTO.getAuthorityAcquireProve())) {
                obsUtil.delete(copyrightDTO.getAuthorityAcquireProve());
            }
            // 删除权力归属证明附件
            if (StringUtils.isNotBlank(copyrightDTO.getAuthorityAscriptionProve())) {
                obsUtil.delete(copyrightDTO.getAuthorityAscriptionProve());
            }
            // 删除文件列表
            if (CollectionUtil.isNotEmpty(copyrightDTO.getFiles())) {
                for (CopyrightDTO.File file : copyrightDTO.getFiles()) {
                    if (StringUtils.isNotBlank(file.getFileUrl())) {
                        obsUtil.delete(file.getFileUrl());
                    }
                }
            }
            // 删除著作人列表的附件
            /*if (CollectionUtil.isNotEmpty(copyrightDTO.getOwners())) {
                for (CopyrightDTO.Owner owner : copyrightDTO.getOwners()) {
                    if (StringUtils.isNotBlank(owner.getBackPhoto())) {
                        ossUtil.delete(owner.getBackPhoto());
                    }
                    if (StringUtils.isNotBlank(owner.getPositivePhoto())) {
                        ossUtil.delete(owner.getPositivePhoto());
                    }
                    if (StringUtils.isNotBlank(owner.getPersonalPhoto())) {
                        ossUtil.delete(owner.getPersonalPhoto());
                    }
                    if (StringUtils.isNotBlank(owner.getCertificatesPhoto())) {
                        ossUtil.delete(owner.getCertificatesPhoto());
                    }
                    if (StringUtils.isNotBlank(owner.getEntrustProve())) {
                        ossUtil.delete(owner.getEntrustProve());
                    }
                }
            }*/
        }
        return draftMapper.deleteById(id) > 0;
    }

    @Override
    public Integer saveDraft(DraftDTO draft) {
        Draft d = new Draft();
        BeanUtil.copyProperties(draft, d, true);
        this.saveOrUpdate(d);
        return d.getId();
    }
}
