package com.fzm.common.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.EntrustShelf;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.EntrustShelfMapper;
import com.fzm.common.service.EntrustShelfService;
import com.fzm.common.service.NftService;
import com.fzm.common.utils.JwtUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * @author tangtuo
 * @date 2021/7/5 15:09
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class EntrustShelfServiceImpl extends ServiceImpl<EntrustShelfMapper, EntrustShelf> implements EntrustShelfService {

    @Resource
    private HttpServletRequest request;

    @Resource
    private NftService nftService;


    @Override
    public Boolean submit(EntrustShelf entrustShelf) {
        // 查询当前nft是否已提交过申请
        QueryWrapper<EntrustShelf> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("nft_hash", entrustShelf.getNftHash());
        queryWrapper.eq("status", SystemConstant.BOOLEAN_DATA_FALSE);
        if (this.count(queryWrapper) > 0) {
            throw GlobalException.newException(ResultCode.FAILED, "此nft已提交过上架申请,请勿重复提交");
        }
        // 修改当前nft的是否已申请的状态为是
        nftService.updateEntrust(entrustShelf.getNftHash(), SystemConstant.BOOLEAN_DATA_TRUE);
        String authorization = request.getHeader("Authorization");
        Integer userId = JwtUtil.getUserIdFromToken(authorization);
        entrustShelf.setUserId(userId);
        return this.save(entrustShelf);
    }

    @Override
    public Boolean cancel(Integer id) {
        EntrustShelf entrustShelf = this.getById(id);
        if (entrustShelf == null) {
            throw GlobalException.newException(ResultCode.SELECT_FAILED);
        }
        // 已取消
        if (entrustShelf.getStatus().equals(SystemConstant.BOOLEAN_DATA_TRUE)) {
            throw GlobalException.newException(ResultCode.FAILED, "当前申请已取消");
        }
        // 修改当前nft的是否已申请的状态为否
        nftService.updateEntrust(entrustShelf.getNftHash(), SystemConstant.BOOLEAN_DATA_FALSE);
        entrustShelf.setStatus(SystemConstant.BOOLEAN_DATA_TRUE);
        return this.updateById(entrustShelf);
    }

    @Override
    public PageInfo<EntrustShelf> getPages(Integer pageNum, Integer pageSize, String telephone, String name, String from, String to) {
        PageHelper.startPage(pageNum, pageSize);
        QueryWrapper<EntrustShelf> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.eq("name", name);
        }
        if (StringUtils.isNotBlank(telephone)) {
            queryWrapper.eq("telephone", telephone);
        }
        if (StringUtils.isNotBlank(from)) {
            queryWrapper.ge("create_date", DateUtil.parse(from + " 00:00:00", "yyyy-MM-dd HH:mm:ss"));

        }
        if (StringUtils.isNotBlank(to)) {
            queryWrapper.le("create_date", DateUtil.parse(to + " 23:59:59", "yyyy-MM-dd HH:mm:ss"));

        }
        return new PageInfo<>(this.list(queryWrapper));
    }

    @Override
    public EntrustShelf getByNftHash(String nftHash) {
        QueryWrapper<EntrustShelf> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("nft_hash", nftHash);
        queryWrapper.eq("status", SystemConstant.BOOLEAN_DATA_FALSE);
        return this.getOne(queryWrapper);
    }
}
