package com.fzm.common.utils;

import cn.hutool.core.util.IdUtil;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.properties.ObsProperties;
import com.obs.services.ObsClient;
import com.obs.services.model.DeleteObjectResult;
import com.obs.services.model.ObsObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;

/**
 * @author tangtuo
 * @date 2021/6/29 16:07
 *
 * <p>
 * 华为云对象存储服务工具类
 * </p>
 */
@Slf4j
@Component
public class ObsUtil {

    @Resource
    private ObsClient obsClient;

    @Resource
    private ObsProperties obsProperties;

    @Resource
    private HttpServletResponse response;

    /**
     * 上传对象
     *
     * @param file
     * @return
     */
    public String putObject(MultipartFile file) throws IOException {
        String objectName = IdUtil.simpleUUID().concat("/").concat(file.getOriginalFilename().replaceAll(" ", ""));
        obsClient.putObject(obsProperties.getBucket(), objectName, file.getInputStream());
        return String.format("https://%s.%s/%s", obsProperties.getBucket(), obsProperties.getEndPoint(), objectName);
    }


    /**
     * 下载对象
     *
     * @param fileUrl  文件地址
     * @param fileName 文件名
     */
    public void downloadFile(String fileUrl, String fileName) {
        try {
            // 文件名以附件的形式下载
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage(), e);
            throw GlobalException.newException(ResultCode.FILE_DOWNLOAD_ERROR, e.getMessage());
        }
        String objectName = fileUrl.substring(String.format("https://%s.%s/", obsProperties.getBucket(), obsProperties.getEndPoint()).length());
        ObsObject object = obsClient.getObject(obsProperties.getBucket(), objectName);
        try {
            // 读取文件内容。
            InputStream inputStream = object.getObjectContent();
            BufferedInputStream in = new BufferedInputStream(inputStream);// 把输入流放入缓存流
            ServletOutputStream outputStream = response.getOutputStream();
            BufferedOutputStream out = new BufferedOutputStream(outputStream);// 把输出流放入缓存流
            byte[] buffer = new byte[1024];
            int len;
            while ((len = in.read(buffer)) != -1) {
                out.write(buffer, 0, len);
            }
            out.flush();
            out.close();
            in.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw GlobalException.newException(ResultCode.FILE_DOWNLOAD_ERROR, e.getMessage());
        }
    }


    /**
     * 删除对象
     *
     * @param fileUrl
     * @return
     */
    public boolean delete(String fileUrl) {
        String objectName = fileUrl.substring(String.format("https://%s.%s/", obsProperties.getBucket(), obsProperties.getEndPoint()).length());
        DeleteObjectResult result = obsClient.deleteObject(obsProperties.getBucket(), objectName);
        return !result.isDeleteMarker();
    }


    public static void main(String[] args) {
        String endPoint = "obs.cn-east-3.myhuaweicloud.com";
        String ak = "HIULSHJZJOTWB72Z38ZB";
        String sk = "GI0oWEGRJuMKmPXJq2fCmuMpFPIITLmjeCQxo2tA";
        String bucketName = "filmchain-file";
        // 创建ObsClient实例
        ObsClient obsClient = new ObsClient(ak, sk, endPoint);
        String url = "https://filmchain-file.obs.cn-east-3.myhuaweicloud.com/8792bab6a7fe4bb58e64b2c4f54ab220/kubernetes(k8s)课程.pdf";
        String objectName = url.substring(String.format("https://%s.%s/", bucketName, endPoint).length());
        System.out.println(objectName);
        DeleteObjectResult deleteObjectResult = obsClient.deleteObject(bucketName, objectName);
        System.out.println(deleteObjectResult);
//        String objectName = IdUtil.simpleUUID().concat("/").concat("kubernetes(k8s)课程.pdf");
//        PutObjectResult putObjectResult = obsClient.putObject(bucketName, objectName, new File("D:\\资料\\k8s\\课程笔记\\kubernetes(k8s)课程.pdf"));
//        System.out.println(putObjectResult);
//        System.out.println(String.format("https://%s.%s/%s", bucketName, endPoint, objectName));
    }


}
