package com.fzm.common.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.properties.SmsProperties;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author tangtuo
 * @date 2021/6/28 16:02
 */
@Component
public class SmsUtil {

    @Resource
    private SmsProperties smsProperties;


    public static String getStringToSignOfStr(Map<String, Object> map) {
        TreeMap<String, String> treeMap = new TreeMap<>();
        Iterator<String> iterator = map.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            treeMap.put(key, map.get(key).toString());
        }
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, String> entry : treeMap.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            sb.append(key).append("=").append(value).append("&");
        }
        if (sb.length() > 1) {
            return sb.deleteCharAt(sb.length() - 1).toString();
        } else {
            return "";
        }
    }

    public static String getTimestamp() {
        return DateUtil.format(new Date(), "yyyyMMddHHmmss");
    }

    public static String getSign(String appkey, String appSecret, String paramStr, String date) {
        return SecureUtil.md5(appkey + paramStr + appSecret + date).toUpperCase();
    }

    /**
     * @param codetype 模板类型
     * @param mobile   手机号
     * @return
     */
    public Boolean sendSms(String codetype, String mobile) {
        String timestamp = getTimestamp();
        HashMap<String, Object> params = new HashMap<>();
        params.put("mobile", mobile);
        params.put("codetype", codetype);
        params.put("param", "FzmRandom4");
        String paramStr = getStringToSignOfStr(params);
        System.out.println(paramStr);
        String sign = getSign(smsProperties.getAppKey(), smsProperties.getAppSecret(), paramStr, timestamp);
        HttpResponse response = HttpRequest.post(smsProperties.getSendSmsUrl()).
                header("FZM-Ca-Timestamp", timestamp).
                header("FZM-Ca-AppKey", smsProperties.getAppKey()).
                header("FZM-Ca-Signature", sign).form(params).execute();
        if (response == null || StringUtils.isBlank(response.body())) {
            return false;
        }
        JSONObject jsonObject = JSONUtil.parseObj(response.body());
        if (jsonObject.get("code", Integer.class) != HttpStatus.HTTP_OK) {
            throw GlobalException.newException(ResultCode.CODE_ERROR, jsonObject.getStr("message"));
        }
        return true;
    }

    /**
     * @param codetype 模板类型,和发短信的一样
     * @param mobile   手机号
     * @param code     验证码
     * @param type     sms/email/voice
     * @return
     */
    public Boolean validateCode(String codetype, String mobile, String code, String type) {
        // todo 删除
        if ("1234".equals(code)) {
            return true;
        }
        String timestamp = getTimestamp();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("t", type);
        paramMap.put("codetype", codetype);
        paramMap.put("code", code);
        paramMap.put("guide", "1");
        paramMap.put("mobile", mobile);
        String str = getStringToSignOfStr(paramMap);
        String sign = getSign(smsProperties.getAppKey(), smsProperties.getAppSecret(), str, timestamp);
        HttpResponse response = HttpRequest.post(smsProperties.getValidateCodeUrl()).
                header("FZM-Ca-Timestamp", timestamp).
                header("FZM-Ca-AppKey", smsProperties.getAppKey()).
                header("FZM-Ca-Signature", sign).form(paramMap).execute();
        if (response == null || StringUtils.isBlank(response.body())) {
            return false;
        }
        JSONObject jsonObject = JSONUtil.parseObj(response.body());
        if (jsonObject.get("code", Integer.class) != HttpStatus.HTTP_OK) {
            throw GlobalException.newException(ResultCode.CODE_ERROR, jsonObject.getStr("message"));
        }
        return true;
    }

/*    public static void main(String[] args) {
        String timestamp = getTimestamp();
        HashMap<String, Object> params = new HashMap<>();
        params.put("mobile", "17620078872");
        params.put("codetype", "quick");
        params.put("param", "FzmRandom4");
        String paramStr = getStringToSignOfStr(params);
        System.out.println(paramStr);
        String sign = getSign(APP_KEY, APP_SECRET, paramStr, timestamp);
        HttpRequest request = HttpRequest.post("http://118.31.52.32/send/sms2").
                header("FZM-Ca-Timestamp", timestamp).
                header("FZM-Ca-AppKey", APP_KEY).
                header("FZM-Ca-OS", "h5").
                header("FZM-Ca-Signature", sign).form(params);
        System.out.println(request.getUrl());

        System.out.println(request.execute().body());

        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("t", "sms");
        paramMap.put("codetype", "quick");
        paramMap.put("code", "8102");
        paramMap.put("guide", "1");
        paramMap.put("mobile", "17620078872");
        String str = getStringToSignOfStr(paramMap);
        String sign1 = getSign(APP_KEY, APP_SECRET, str, timestamp);
        HttpResponse h5 = HttpRequest.post("http://118.31.52.32/validate/code").
                header("FZM-Ca-Timestamp", timestamp).
                header("FZM-Ca-AppKey", APP_KEY).
                header("FZM-Ca-OS", "h5").
                header("FZM-Ca-Signature", sign1).form(paramMap).execute();
        System.out.println(h5.body());
    }*/
}
