package com.fzm.portal.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.CopyrightApply;
import com.fzm.common.entity.dto.CopyrightDTO;
import com.fzm.common.entity.vo.CopyrightCertificateVo;
import com.fzm.common.entity.vo.CopyrightStateVo;
import com.fzm.common.entity.vo.CopyrightVo;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.CopyrightApplyService;
import com.fzm.common.utils.JwtUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.ExecutionException;

/**
 * @author tangtuo
 * @date 2021/12/9 14:31
 */
@RestController
@Api(tags = "版权申请")
@RequestMapping("/copyright/apply")
public class CopyrightApplyController {

    @Resource
    private CopyrightApplyService copyrightApplyService;

    @Authentication
    @PostMapping("/submit")
    @ApiOperation("/提交申请")
    public ResponseModel<Integer> submit(@Validated @RequestBody CopyrightDTO copyrightDTO) {
        Integer copyrightId = copyrightApplyService.submit(copyrightDTO);
        return ResponseModel.success(copyrightId);
    }


    @Authentication
    @GetMapping("/list")
    @ApiOperation(value = "查询版权登记列表")
    public ResponseModel<List<CopyrightApply>> list(@RequestHeader(value = "Authorization") String token, @ApiParam("状态 0-全部 1-进行中 2-已完成") @RequestParam Integer state) {
        Integer userId = JwtUtil.getUserIdFromToken(token);
        List<CopyrightApply> list = copyrightApplyService.list(userId, state);
        return ResponseModel.success(list);
    }

    @Authentication
    @PostMapping(value = "/withdraw")
    @ApiOperation(value = "撤回")
    public ResponseModel<Boolean> withdraw(@RequestParam Integer id) {
        Boolean result = copyrightApplyService.withdraw(id);
        return ResponseModel.success(result);
    }

    //@Authentication
    @PostMapping(value = "/delete")
    @ApiOperation(value = "删除")
    public ResponseModel<Boolean> delete(@RequestParam Integer id) {
        Boolean result = copyrightApplyService.delete(id);
        return ResponseModel.success(result);
    }

    @GetMapping(value = "/detail")
    @ApiOperation(value = "获取详情")
    public ResponseModel<CopyrightVo> getById(@RequestParam Integer id) throws ExecutionException, InterruptedException {
        CopyrightVo copyrightVo = copyrightApplyService.getDetail(id);
        return ResponseModel.success(copyrightVo);
    }

    @Authentication
    @PostMapping(value = "/update")
    @ApiOperation(value = "编辑")
    public ResponseModel<Boolean> update(@RequestBody CopyrightDTO copyrightDTO) {
        return ResponseModel.success(copyrightApplyService.update(copyrightDTO));
    }

    @GetMapping(value = "/content")
    @ApiOperation(value = "获取json串")
    public ResponseModel<String> getContent(@RequestParam Integer id) {
        String content = copyrightApplyService.getById(id).getContent();
        return ResponseModel.success(content);
    }

    @GetMapping("evidence/count")
    @ApiOperation(value = "获取存证数量")
    public ResponseModel<Integer> getSuccessCount() {
        int count = copyrightApplyService.getSuccessCount();
        return ResponseModel.success(count);
    }


    @GetMapping(value = "/state")
    @ApiOperation(value = "获取版权状态")
    public ResponseModel<CopyrightStateVo> getCopyrightState(@RequestParam Integer id) {
        CopyrightApply copyright = copyrightApplyService.getById(id);
        if (copyright == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "请核对后再重试");
        }
        CopyrightStateVo copyrightStateVo = new CopyrightStateVo(CopyrightApplyState.getByState(copyright.getRegisterState()), copyright.getRejectReason());
        return ResponseModel.success(copyrightStateVo);
    }


    @GetMapping("/certificate/{id}")
    @ApiOperation(value = "查看版权存证证书")
    public ResponseModel<CopyrightCertificateVo> getCertificate(@PathVariable Integer id){
        CopyrightCertificateVo copyrightCertificateVo = copyrightApplyService.getCertificate(id);
        return ResponseModel.success(copyrightCertificateVo);
    }

}
