package com.fzm.portal.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.enums.FileSuffix;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.utils.ObsUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;

/**
 * @author tangtuo
 * @date 2021/12/13 17:26
 */
@Api(tags = "版权申请附件管理")
@Authentication
@RestController
@RequestMapping(value = "/copyright/file")
public class CopyrightFileController {

    private static final long maxSize = 1024 * 1024 * 200;

    @Resource
    private ObsUtil obsUtil;

    @PostMapping("/upload")
    @ApiOperation(value = "上传版权申请附件")
    public ResponseModel<String> upload(MultipartFile file) throws IOException {
        // 文件最大支持200m
        if (file.getSize() > maxSize) {
            throw GlobalException.newException(ResultCode.FILE_UPLOAD_ERROR, "单个文件最大支持200M");
        }
        String fileName = file.getOriginalFilename();
        String fileSuffix = fileName.substring(fileName.lastIndexOf(".") + 1);
        if (!FileSuffix.validFileType(fileSuffix)) {
            throw GlobalException.newException(ResultCode.FILE_UPLOAD_ERROR, "不支持当前上传的文件类型");
        }
        String url = obsUtil.putObject(file);
        return ResponseModel.success(url);
    }
}
