package com.fzm.portal.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.CopyrightOwner;
import com.fzm.common.enums.OwnerType;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.CopyrightOwnerService;
import com.fzm.common.utils.JwtUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/12/13 10:16
 */
@Authentication
@RestController
@RequestMapping("/copyright/owner")
@Api(tags = "著作权人")
public class CopyrightOwnerController {

    @Resource
    private CopyrightOwnerService copyrightOwnerService;

    @PostMapping(value = "/add")
    @ApiOperation(value = "新增著作权人")
    public ResponseModel<Integer> add(@RequestHeader String Authorization, @Validated @RequestBody CopyrightOwner owner) {
        // 如果著作权人是个人的话,身份证正反面照片 著作权人手持身份证照片等信息必传
        if (owner.getType().equals(OwnerType.PERSON.getType())) {
            if (StringUtils.isAnyBlank(owner.getBackPhoto(), owner.getPositivePhoto(), owner.getPersonalPhoto())) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
            }
        } else {
            // 著作权人是法人的话, 证件照片必传
            if (StringUtils.isBlank(owner.getCertificatesPhoto())) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
            }
        }
        Integer userId = JwtUtil.getUserIdFromToken(Authorization);
        owner.setUserId(userId);
        copyrightOwnerService.save(owner);
        return ResponseModel.success(owner.getId());
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "获取我的著作权人列表")
    public ResponseModel<List<CopyrightOwner>> list(@RequestHeader(value = "Authorization") String token) {
        Integer userId = JwtUtil.getUserIdFromToken(token);
        List<CopyrightOwner> list = copyrightOwnerService.getByUserId(userId);
        return ResponseModel.success(list);
    }

    @PostMapping("/update")
    @ApiOperation(value = "修改")
    public ResponseModel<Boolean> update(@Validated @RequestBody CopyrightOwner owner) {
        return ResponseModel.success(copyrightOwnerService.updateById(owner));
    }

    @PostMapping("/delete")
    @ApiOperation(value = "删除")
    public ResponseModel<Boolean> delete(@RequestParam Integer id) {
        return ResponseModel.success(copyrightOwnerService.removeById(id));
    }

}
