package com.fzm.portal.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.Draft;
import com.fzm.common.entity.dto.DraftDTO;
import com.fzm.common.entity.vo.CopyrightVo;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.DraftService;
import com.fzm.common.utils.JwtUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/12/10 10:19
 */
@Authentication
@RestController
@RequestMapping("/Draft")
@Api(tags = "草稿管理")
public class DraftController {

    @Resource
    private DraftService draftService;

    @PostMapping("/save")
    @ApiOperation(value = "保存草稿")
    public ResponseModel<Integer> save(@RequestHeader(value = "Authorization") String token, @RequestBody DraftDTO draft) {
        draft.setUserId(JwtUtil.getUserIdFromToken(token));
        Integer id = draftService.saveDraft(draft);
        return ResponseModel.success(id);
    }

    @GetMapping("/list")
    @ApiOperation(value = "查询我的草稿列表")
    public ResponseModel<List<Draft>> list(@RequestHeader(value = "Authorization") String token) {
        Integer userId = JwtUtil.getUserIdFromToken(token);
        List<Draft> list = draftService.getByUserId(userId);
        return ResponseModel.success(list);
    }

    @PostMapping("/delete")
    @ApiOperation(value = "删除草稿记录")
    public ResponseModel<Boolean> delete(@RequestParam Integer id) {
        Boolean result = draftService.delete(id);
        return ResponseModel.success(result);
    }


    @GetMapping("/detail")
    @ApiOperation(value = "获取草稿详情")
    public Draft getDetail(Integer id) {
        return draftService.getById(id);
    }

}
