package com.fzm.portal.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.User;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.properties.SmsProperties;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.SmsUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * @author tangtuo
 * @date 2021/6/23 15:56
 */
@RestController
@RequestMapping(value = "/verificationCode")
@Api(tags = "验证码管理")
public class VerificationCodeController {


    @Resource
    private UserService userService;

    @Resource
    private SmsProperties smsProperties;

    @Resource
    private SmsUtil smsUtil;


    @PostMapping(value = "/send/sms")
    @ApiOperation(value = "发送短信")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "telephone", value = "手机号"),
            @ApiImplicitParam(name = "codetype", value = "短信模板, 1:登录短信 2:修改密码 3:修改手机号"),
    })
    public ResponseModel<Boolean> sendSms(@RequestParam String telephone, @RequestParam Integer codetype) {
        return ResponseModel.success(smsUtil.sendSms(getCodeType(codetype), telephone));
    }

    @Authentication
    @ApiOperation("发送转让nft的短信")
    @PostMapping("/send/Sms/transferNft")
    public ResponseModel<Boolean> sendTransferNftSms() {
        User user = userService.getUserByToken();
        if (user == null || StringUtils.isBlank(user.getTelephone())) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, "当前用户的手机号为空");
        }
        Boolean result = smsUtil.sendSms(smsProperties.getTransferNftMessageCodetype(), user.getTelephone());
        return ResponseModel.success(result);
    }


    private String getCodeType(Integer type) {
        switch (type) {
            case 1:
                return smsProperties.getLoginMessageCodetype();
            default:
                throw GlobalException.newException(ResultCode.CODE_ERROR);
        }
    }

}
